;******************************************************************************
;
;       minirgb 16F84
;       Program = Serielle Waage
;       Revision Date: 20.07.07
;       Version 0.51
;       
;
;******************************************************************************
;******************************************************************************
;******************************************************************************
;=========================
;=       INFO            =
;=========================


;******************************************************************************
;
;       Quarz 4.00000 MHz
;
;******************************************************************************

;******************************************************************************
;******************************************************************************
;=========================
;=   PORT CONNECTIONS    =
;=========================


;******************************************************************************
;       
;       PORTA0 = COMTX
;                            R0  R1  R2  R3
;       PORTA1 = DISPLAY 0 / f   g   e   d
;       PORTA2 = DISPLAY 1 / f   g   e   d
;       PORTA3 = DISPLAY 2 / f   g   e   d
;       PORTA4 = DISPLAY 3 / f   g   e   d
;
;******************************************************************************

;******************************************************************************
;       
;       PORTB0 = ROW 0
;       PORTB1 = ROW 1
;       PORTB2 = ROW 2
;       PORTB3 = ROW 3        
;                            R0  R1  R2  R3
;       PORTB4 = DISPLAY 0 / a   b   c   nc
;       PORTB5 = DISPLAY 1 / a   b   c   nc
;       PORTB6 = DISPLAY 2 / a   b   c   nc
;       PORTB7 = DISPLAY 3 / a   b   c   nc
;
;******************************************************************************

;********************************7seg connections in MBI5016*******************
;
;       Bit0 f
;       Bit1 g
;       Bit2 a
;       Bit3 b
;       
;       Bit4 c
;       Bit5 nc
;       Bit6 d
;       Bit7 e
;       
;
;****************************************************************************** 

;********************************7seg decoder**********************************
;
;       Bit0 db/nc
;       Bit1 c
;       Bit2 d
;       Bit3 e
;       
;       Bit4 g
;       Bit5 f
;       Bit6 a
;       Bit7 b
;       
;
;****************************************************************************** 


;******************************************************************************
;******************************************************************************

;=========================
;=        DEFINES        =
;=========================

;******************************************************************************
;F=Flag
;B=Bitvar for bit defines
;S=static
;T=temporary
;LA=low active       
;P=pointer var
;ADR=adressdefine
;BUT=buttondefine
;F_TL1_=temporary bit in stack level 1 function (no use in L2 function call)
;******************************************************************************


;******************************************************************************

;********** Port A Defines
;*************************

#Define         COMTX           PORTA,0         ;
#Define         PABIT1          PORTA,1         ;
#Define         PABIT2          PORTA,2         ;
#Define         PABIT3          PORTA,3         ;
#Define         PABIT4          PORTA,4         ;


;********** Port B Defines
;*************************

#Define         PBBIT0          PORTB,0         ;
#Define         PBBIT1          PORTB,1         ;
#Define         PBBIT2          PORTB,2         ;
#Define         PBBIT3          PORTB,3         ;
#Define         PBBIT4          PORTB,4         ;
#Define         PBBIT5          PORTB,5         ;
#Define         PBBIT6          PORTB,6         ;
#Define         PBBIT7          PORTB,7         ;


;** Hardware control Flags
;*************************

#Define         F_CLKTRIG       HWCTRL,0        ;
#Define         F_DISPTRIG      HWCTRL,1        ;
#Define         HBIT2           HWCTRL,2        ;
#Define         HBIT3           HWCTRL,3        ;
#Define         HBIT4           HWCTRL,4        ;
#Define         HBIT5           HWCTRL,5        ;
#Define         HBIT6           HWCTRL,6        ;
#Define         HBIT7           HWCTRL,7        ;

;** Software control Flags
;*************************

#Define         SBIT0           SWCTRL,0        ;
#Define         SBIT1           SWCTRL,1        ;
#Define         SBIT2           SWCTRL,2        ;
#Define         SBIT3           SWCTRL,3        ;
#Define         SBIT4           SWCTRL,4        ;
#Define         SBIT5           SWCTRL,5        ;
#Define         SBIT6           SWCTRL,6        ;
#Define         SBIT7           SWCTRL,7        ;

;****** Additional defines
;*************************



;************ DSIPLAY Defs
;*************************


;************** RS232 Defs
;*************************

#Define         COMSTIME_D      d'32'           ;Sendtiming (ermittelt durch messung mit 
                                                ;stopwatch (10 mal (9600baud) 104us)
                                                ; -> 1,04ms fr ein paket @ 4Mhz

;******************************************************************************
;******************************************************************************
;******************************************************************************
;=========================
;=        MACROS         =
;=========================


;******************************************************************************

;************** Bankselect
;*************************

BANK0_          macro
                bcf             STATUS,5
                bcf             STATUS,6
                endm

BANK1_          macro
                bsf             STATUS,5
                bcf             STATUS,6
                endm

;*************** Interrupt
;*************************

PUSH_           macro
                movwf           ISR_W
                swapf           STATUS,w
                movwf           ISR_S
                endm

POP_            macro
                swapf           ISR_S,w
                movwf           STATUS
                swapf           ISR_W,f
                swapf           ISR_W,w
                endm

;******************************************************************************
;******************************************************************************
;******************************************************************************


;************************************************************** SOFTWARE STRUCT 
;******************************************************************************

;.----------------------------------------------------------------------.
;|                                                                      |
;|    MAIN:_START    Software struct of serwaage.asm                    |
;|                                                                      |
;'-o-----------------------------------------o--------------o-----------'
;  |                                         |              |
;  |                                         |              |
;  |                                         |              |
;  |                                         |              |
;  |                                         |              |
;  |                                         |              |
;  |                                         |            .-o----------.
;  |                                         |            |            |
;  |                                         |            | COMSENDDAT_|
;  |                                         |            |           3|
;  |                                         |            '-o----------'
;  |                                         |              |
;  |                                         |              |
;  |                                         |              |
;  |                                         |              |
;.-o---------.                             .-o----------. .-o-------.
;|           |                             |            | |         |
;| GETDIGITS_|                             | TRANSLLATE_| | COMSEND_|
;|          2|                             |           2| |        2|
;'-o-o-o-----'                             '-o----------' '-o-------'
;  | | |                                     |              |
;  | | |                                     |              |
;  | | |                                     |              |
;  | | |                                     |              |
;  | | |                                     |              |
;  | | |                                     |              |
;  | | |                                     |              |
;  | | |                                     |              |
;  | | |                                     |              |
;  | | |                                     |              |
;  | | |                                     |              |
;  | | |                                     |              |
;  | | |                                     |              |
;  | | |                                     |              |
;  | | |                                     |              |
;  | | '----------------------.              |              |
;  | |                        |              |              |
;  | '------------.           |              |              |
;  |              |           |              |              |
;.-o----------. .-o-------. .-o----------. .-o---------.  .-o---------.
;|            | |         | |            | |           |  |           |
;| GETSIGNALS_| | GETWAIT_| | SORTDIGITS_| | SEARCHVAL_|  | COMTIMING_|
;|           1| |        1| |           1| |          1|  |          1|
;'------------' '---------' '------------' '-----------'  '-----------'


;=========================
;=        CODE           =
;=========================



;*************************************************************** Include  Files 
;******************************************************************************

  LIST    P=PIC16F84
#INCLUDE "P16F84.INC"
;#define __16F84


;******************************************************************** Variables 
;******************************************************************************

;****** Standard Variables
;*************************

HWCTRL    equ    h'0c' ;
SWCTRL    equ    h'0d' ;help bits
ISR_W     equ    h'0e' ;
ISR_S     equ    h'0f' ;isr bits

;*********** LCD Variables
;*************************


;********** WAIT Variables
;*************************

ZEIT1     equ    h'10' ;
ZEIT2     equ    h'11' ;
ZEIT3     equ    h'12' ;
ZEIT4     equ    h'13' ;counter in main


;********* Other Variables
;*************************

ISR_TMR   equ    h'14' ;prescaler in interrupt
DEC7S     equ    h'15' ;
VAL       equ    h'16' ;
TMP1      equ    h'17' ;

VARIABLE     COM=h'18' ;

COMCNT    equ    COM   ;
COM++                  ;
COMSEND   equ    COM   ;
COM++                  ;

VARIABLE     USE=COM   ; 

DIGITFA   equ    USE   ;
USE++
DIGITGB   equ    USE   ;
USE++
DIGITEC   equ    USE   ;
USE++
DIGITDdp  equ    USE   ;
USE++

DIGITS0   equ    USE   ;
USE++
DIGITS1   equ    USE   ;
USE++
DIGITS2   equ    USE   ;
USE++
DIGITS3   equ    USE   ;
USE++


ASCI0     equ    USE   ;
USE++
ASCI1     equ    USE   ;
USE++
ASCI2     equ    USE   ;
USE++
ASCI3     equ    USE   ;
USE++

SIGNALS   equ    USE   ;temp
USE++
DIGHLP    equ    USE   ;
USE++
SEARCHVAL equ    USE   ;
USE++

;******************************************************************** DATAVIEW 
;*****************************************************************************
;
;  .----------------------------------------------------------------.
;  |                  serwaage dataview                             |
;  |.--------------------------------------------------------------.|
;  ||                                                              ||
;  ||.------------------------------------------------------------.||
;  |||                                                            |||
;  ||| DIGFA  F3  F2  F1  F0  A3  A2  A1  A0                      |||
;  ||| DIGGB  G3  G2  G1  G0  B3  B2  B1  B0          GETSIGNALS_ |||
;  ||| DIGEC  E3  E2  E1  E0  C3  C2  C1  C0                      |||
;  ||| DIGDdp D3  D2  D1  D0 dp3 dp2 dp1 dnc                      |||
;  ||'----------------------------------------------------o-------'||
;  ||                                                     |        ||
;  ||                                              .------o------. ||
;  ||  DIG0   nc  G0  F0  E0  D0  C0  B0  A0       |             | ||
;  ||  DIG1  dp1  G1  F1  E1  D1  C1  B1  A1 <-----o  SORTDIGITS_| ||
;  ||  DIG2  dp2  G2  F2  E2  D2  C2  B2  A2       |             | ||
;  ||  DIG3  dp3  G3  F3  E3  D3  C3  B3  A3       '-------------' ||
;  |'-----------------------------------------------------o--------'|
;  |        .------------------------------.              |         |
;  |        |    SEARCHVALUE_  translation |              |         |
;  |        |                     table    |       .------o-----.   |
;  |        |  DIGx  GFEDCBA               |       |            |   |
;  |        |  0x3F 00111111  0            | <-----o  TRANSLATE_|   |
;  |        |  0x06 00000110  1            |       |            |   |
;  |        |  0x5B 01011011  2            |       '------------'   |
;  |        |  0x4F 01001111  3            |                        |
;  |        |  0x66 01100110  4            |                        |
;  |        |  0x6D 01101101  5            |                        |
;  |        |  0x7D 01111101  6            |                        |
;  |        |  0x07 00000111  7            |                        |
;  |        |  0x7F 01111111  8            |                        |
;  |        |  0x6F 01101111  9            |                        |
;  |        '-----------------o------------'                        |
;  |                          |                                     |
;  |                          |                                     |
;  |                          |                                     |
;  |                          |                                     |
;  |        .------------------------------.     .----------------. |
;  |        |                 |            |     |                | |
;  |        |  ASC0       <---o            |     |                | |
;  |        |  ASC1       <---o            |     |                | |
;  |        |  ASC2       <---o            o---->|   SENDDATA_    | |
;  |        |  ASC3       <---o            |     |                | |
;  |        |                              |     |                | |
;  |        '------------------------------'     '----------------' |
;  '----------------------------------------------------------------'



;****************************************************** Restet/Interrupt Vektor
;******************************************************************************
org             .0
                goto            _INIT           ;Reset Vektor
org             .4
                goto            _ISR            ;Interrupt Service Routine

;******************************************************************* INIT START
;******************************************************************************
_INIT           
                BANK1_                          ;change to Bank 1

;Port A
 
                movlw           b'00011110'     ;0->Output
                movwf           TRISA           ;
 
;Port B
 
                movlw           b'11111111'     ;0->Output
                movwf           TRISB           ;
 
;Option Reg 

                movlw           b'10000011'     ;Prescaler on(Bit3) 0 bis 2 Prescaler(auf 2^4(teilung 16)
                movwf           OPTION_REG      ; 

;Intcon Reg
     
                movlw           b'00100000'     ;Interruptkontrolle bit5 tmr0 int on/off, bit4 rb0 int on/off
                movwf           INTCON          ;

                BANK0_                          ;change to Bank 0

;************************************************************** Initialisierung
;******************************************************************************

                clrf            HWCTRL          ;
                clrf            SWCTRL          ;

;***************************************** ONE TIMES CALLED FUNCTIONS AND INITS
;******************************************************************************

                call            COMINIT_

                movlw           '0'
                movwf           DIGITS0
                movlw           ' '
                movwf           DIGITS1
                movwf           DIGITS2
                movwf           DIGITS3


  
;**************************************************************** START OF MAIN
;******************************************************************************

_START
                call            GETDIGITS_

                call            TRANSLATE_

                call            COMSENDDAT_
               
                movlw		.1
		movwf		ZEIT3

 
MESS3_		movlw		.255
		movwf		ZEIT2

MESS2_		movlw		.255
		movwf		ZEIT1
		
         
MESS1_          decfsz		ZEIT1,F	
		goto		MESS1_
		decfsz		ZEIT2,F
		goto 		MESS2_
		decfsz		ZEIT3,F
		goto 		MESS3_


_END            goto            _START

;******************************************************************************
;****************************************************************** END OF MAIN

;************************************************************************ INITS
;******************************************************************************




;******************************************************************************
;***************************************************************** END OF INITS

;*************************************************************** MAIN FUNCTIONS
;******************************************************************************

;------------------------------------------------------------------------------
;Cat:         HW Func / GETDIGITS_
;------------------------------------------------------------------------------
;Level:       2
;------------------------------------------------------------------------------
;Task:        
;------------------------------------------------------------------------------
;Functions:   GETDIGITS_
;Based on:    GETSIGNALS_
;             SORTDIGITS_
;------------------------------------------------------------------------------
;Vars:          DIGITS0         ; nc G0 F0 E0 D0 C0 B0 A0
;               DIGITS1         ; p1 G1 F1 E1 D1 C1 B1 A1
;               DIGITS2         ; p2 G2 F2 E2 D2 C2 B2 A2
;               DIGITS3         ; p3 G3 F3 E3 D3 C3 B3 A3
;
;Internal:      DIGITFA         ; F3 F2 F1 F0 A3 A2 A1 A0            if RB0 = 0
;               DIGITGB         ; G3 G2 G1 G0 B3 B2 B1 B0            if RB1 = 0
;               DIGITEC         ; E3 E2 E1 E0 C3 C2 C1 C0            if RB2 = 0
;               DIGITDdp        ; D3 D2 D1 D0 p3 p2 p1 nc            if RB3 = 0
;------------------------------------------------------------------------------
;Bits:        
;Internal:    
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------
GETDIGITS_

_GET0H          btfsc           PORTB,0
                goto            _GET0H

_GET0L          btfss           PORTB,0
                goto            _GET0L

                call            GETWAIT_

                call            GETSIGNALS_

                movf            SIGNALS,w
                movwf           DIGITFA

_GET1H          btfsc           PORTB,1
                goto            _GET1H

_GET1L          btfss           PORTB,1
                goto            _GET1L

                call            GETWAIT_

                call            GETSIGNALS_

                movf            SIGNALS,w
                movwf           DIGITGB

_GET2H          btfsc           PORTB,2
                goto            _GET2H

_GET2L          btfss           PORTB,2
                goto            _GET2L

                call            GETWAIT_

                call            GETSIGNALS_

                movf            SIGNALS,w
                movwf           DIGITEC

_GET3H          btfsc           PORTB,3
                goto            _GET3H

_GET3L          btfss           PORTB,3
                goto            _GET3L

                call            GETWAIT_

                call            GETSIGNALS_

                movf            SIGNALS,w
                movwf           DIGITDdp

                call            SORTDIGITS_

                return

;------------------------------------------------------------------------------
;Cat:         HW Func / GETSIGNALS
;------------------------------------------------------------------------------
;Level:       1
;------------------------------------------------------------------------------
;Task:        get digit information from ports
;------------------------------------------------------------------------------
;Functions:   GETSIGNALS_
;Based on:    no
;------------------------------------------------------------------------------
;Vars:        SIGNALS
;Internal:    no
;------------------------------------------------------------------------------
;Bits:        no
;Internal:    no
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------

GETSIGNALS_ 
                movf            PORTA,w
                movwf           SIGNALS 
                rlf             SIGNALS,f
                rlf             SIGNALS,f
                rlf             SIGNALS,f
                movf            SIGNALS,w
                andlw           h'F0'
                movwf           SIGNALS
   
                swapf           PORTB,w
                andlw           h'0f'
                xorwf           SIGNALS,f
                return

;------------------------------------------------------------------------------
;Cat:         SW Func / SORTDIGITS_
;------------------------------------------------------------------------------
;Level:       1
;------------------------------------------------------------------------------
;Task:        
;------------------------------------------------------------------------------
;Functions:   SORTDIGITS_
;Based on:    no
;------------------------------------------------------------------------------
;Vars:          DIGITS0         ; nc G0 F0 E0 D0 C0 B0 A0
;               DIGITS1         ; p1 G1 F1 E1 D1 C1 B1 A1
;               DIGITS2         ; p2 G2 F2 E2 D2 C2 B2 A2
;               DIGITS3         ; p3 G3 F3 E3 D3 C3 B3 A3
;
;Internal:      DIGITFA         ; F3 F2 F1 F0 A3 A2 A1 A0            if RB0 = 0
;               DIGITGB         ; G3 G2 G1 G0 B3 B2 B1 B0            if RB1 = 0
;               DIGITEC         ; E3 E2 E1 E0 C3 C2 C1 C0            if RB2 = 0
;               DIGITDdp        ; D3 D2 D1 D0 p3 p2 p1 nc            if RB3 = 0
;------------------------------------------------------------------------------
;Bits:        
;Internal:    
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------
SORTDIGITS_
                clrf            DIGITS0
                clrf            DIGITS1
                clrf            DIGITS2
                clrf            DIGITS3

;digits f and a

                btfss           DIGITFA,0
                bsf             DIGITS0,0

                btfss           DIGITFA,1
                bsf             DIGITS1,0

                btfss           DIGITFA,2
                bsf             DIGITS2,0

                btfss           DIGITFA,3
                bsf             DIGITS3,0


                btfss           DIGITFA,4
                bsf             DIGITS0,5

                btfss           DIGITFA,5
                bsf             DIGITS1,5

                btfss           DIGITFA,6
                bsf             DIGITS2,5

                btfss           DIGITFA,7
                bsf             DIGITS3,5

;digits g and b
                btfss           DIGITGB,0
                bsf             DIGITS0,1

                btfss           DIGITGB,1
                bsf             DIGITS1,1

                btfss           DIGITGB,2
                bsf             DIGITS2,1

                btfss           DIGITGB,3
                bsf             DIGITS3,1


                btfss           DIGITGB,4
                bsf             DIGITS0,6

                btfss           DIGITGB,5
                bsf             DIGITS1,6

                btfss           DIGITGB,6
                bsf             DIGITS2,6

                btfss           DIGITGB,7
                bsf             DIGITS3,6

;digits e and c
                btfss           DIGITEC,0
                bsf             DIGITS0,2

                btfss           DIGITEC,1
                bsf             DIGITS1,2

                btfss           DIGITEC,2
                bsf             DIGITS2,2

                btfss           DIGITEC,3
                bsf             DIGITS3,2


                btfss           DIGITEC,4
                bsf             DIGITS0,4

                btfss           DIGITEC,5
                bsf             DIGITS1,4

                btfss           DIGITEC,6
                bsf             DIGITS2,4

                btfss           DIGITEC,7
                bsf             DIGITS3,4

;digits d and dp

                btfss           DIGITDdp,0
                bsf             DIGITS0,7

                btfss           DIGITDdp,1
                bsf             DIGITS1,7

                btfss           DIGITDdp,2
                bsf             DIGITS2,7

                btfss           DIGITDdp,3
                bsf             DIGITS3,7


                btfss           DIGITDdp,4
                bsf             DIGITS0,3

                btfss           DIGITDdp,5
                bsf             DIGITS1,3

                btfss           DIGITDdp,6
                bsf             DIGITS2,3

                btfss           DIGITDdp,7
                bsf             DIGITS3,3

                return

GETWAIT_
                movlw           .1
                movwf           ZEIT2

_GETW2          movlw           .255
		movwf           ZEIT1
		
_GETW1          decfsz		ZEIT1,F	
		goto		_GETW1
		decfsz		ZEIT2,F
		goto 		_GETW2
                return

;------------------------------------------------------------------------------
;Cat:         SW Func / TRANSLATE_
;------------------------------------------------------------------------------
;Level:       2
;------------------------------------------------------------------------------
;Task:        translate all digit infos to asci format
;------------------------------------------------------------------------------
;Functions:    SEARCHVAL_ 
;Based on:    
;------------------------------------------------------------------------------
;Vars:          DIGITS0
;               DIGITS1
;               DIGITS2
;               DIGITS3
;
;               ACSI0
;               ACSI1
;               ACSI2
;               ACSI3
;Internal:    
;------------------------------------------------------------------------------
;Bits:        
;Internal:    
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------

TRANSLATE_
                movf            DIGITS0,w
                andlw           h'7F'

                movwf           SEARCHVAL
                call            SEARCHVAL_

                movf            SEARCHVAL,w
                movwf           ASCI0



                movf            DIGITS1,w
                andlw           h'7F'

                movwf           SEARCHVAL
                call            SEARCHVAL_

                movf            SEARCHVAL,w
                movwf           ASCI1


                movf            DIGITS2,w
                andlw           h'7F'

                movwf           SEARCHVAL
                call            SEARCHVAL_

                movf            SEARCHVAL,w
                movwf           ASCI2


                movf            DIGITS3,w
                andlw           h'7F'

                movwf           SEARCHVAL
                call            SEARCHVAL_

                movf            SEARCHVAL,w
                movwf           ASCI3

                return

;------------------------------------------------------------------------------
;Cat:         SW Func / SEARCHVAL_
;------------------------------------------------------------------------------
;Level:       1
;------------------------------------------------------------------------------
;Task:        translate a digit info to asci format
;------------------------------------------------------------------------------
;Functions:   
;Based on:    
;------------------------------------------------------------------------------
;Vars:        SEARCHVAL (back in w)
;Internal:    
;------------------------------------------------------------------------------
;Bits:        
;Internal:    
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------
SEARCHVAL_

                movlw           h'00'           ;digits of off
                xorwf           SEARCHVAL,w
                btfss           STATUS,2
                goto            _NEXT0

                movlw           ' '
                movwf           SEARCHVAL
                goto            _SEARCHEND


_NEXT0          movlw           h'3F'           ;digits of 0
                xorwf           SEARCHVAL,w
                btfss           STATUS,2
                goto            _NEXT1

                movlw           '0'
                movwf           SEARCHVAL
                goto            _SEARCHEND


_NEXT1          movlw           h'06'           ;digits of 1
                xorwf           SEARCHVAL,w
                btfss           STATUS,2
                goto            _NEXT2

                movlw           '1'
                movwf           SEARCHVAL
                goto            _SEARCHEND


_NEXT2          movlw           h'5B'           ;digits of 2
                xorwf           SEARCHVAL,w
                btfss           STATUS,2
                goto            _NEXT3

                movlw           '2'
                movwf           SEARCHVAL
                goto            _SEARCHEND


_NEXT3          movlw           h'4F'           ;digits of 3
                xorwf           SEARCHVAL,w
                btfss           STATUS,2
                goto            _NEXT4

                movlw           '3'
                movwf           SEARCHVAL
                goto            _SEARCHEND


_NEXT4          movlw           h'66'           ;digits of 4
                xorwf           SEARCHVAL,w
                btfss           STATUS,2
                goto            _NEXT5

                movlw           '4'
                movwf           SEARCHVAL
                goto            _SEARCHEND


_NEXT5          movlw           h'6D'           ;digits of 5
                xorwf           SEARCHVAL,w
                btfss           STATUS,2
                goto            _NEXT6

                movlw           '5'
                movwf           SEARCHVAL
                goto            _SEARCHEND


_NEXT6          movlw           h'7D'           ;digits of 6
                xorwf           SEARCHVAL,w
                btfss           STATUS,2
                goto            _NEXT7

                movlw           '6'
                movwf           SEARCHVAL
                goto            _SEARCHEND


_NEXT7          movlw           h'07'           ;digits of 7
                xorwf           SEARCHVAL,w
                btfss           STATUS,2
                goto            _NEXT8

                movlw           '7'
                movwf           SEARCHVAL
                goto            _SEARCHEND


_NEXT8          movlw           h'7F'           ;digits of 8
                xorwf           SEARCHVAL,w
                btfss           STATUS,2
                goto            _NEXT9

                movlw           '8'
                movwf           SEARCHVAL
                goto            _SEARCHEND


_NEXT9          movlw           h'6F'           ;digits of 9
                xorwf           SEARCHVAL,w
                btfss           STATUS,2
                goto            _ERR

                movlw           '9'
                movwf           SEARCHVAL
                goto            _SEARCHEND

_ERR
                movlw           '-'
                movwf           SEARCHVAL

_SEARCHEND      return


    



;******************************************************************************
;*********************************************************** END MAIN FUNCTIONS

                ;**************************************
                ;*                                    *
                ;*           *
                ;*                                    *
                ;************************************** 

                ;***************
                ;*             *
                ;*    *
                ;*             *
                ;***************       

;------------------------------------------------------------------------------
;Cat:         SW Func / class xxx /
;------------------------------------------------------------------------------
;Level:       x
;------------------------------------------------------------------------------
;Task:        
;------------------------------------------------------------------------------
;Functions:   
;Based on:    
;------------------------------------------------------------------------------
;Vars:        
;Internal:    
;------------------------------------------------------------------------------
;Bits:        
;Internal:    
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------

;******************************************************************** FUNCTIONS
;******************************************************************************

;------------------------------------------------------------------------------
;Cat:         SW Func / class com / COMSENDDAT_
;------------------------------------------------------------------------------
;Level:       3
;------------------------------------------------------------------------------
;Task:        init com pin
;------------------------------------------------------------------------------
;Functions:     COMSENDDAT_
;Based on:      COMSEND_
;------------------------------------------------------------------------------
;Vars:          DIG4 - DIG0
;Internal:      COMSEND
;------------------------------------------------------------------------------
;Bits:          no
;Internal:      no
;------------------------------------------------------------------------------
;Defines:       no
;Internal:      no
;---------------|---------------|---------------|------------------------------
COMSENDDAT_


		movf		ASCI3,w
		movwf		COMSEND
		call		COMSEND_

		call		COMTIMING_
		call		COMTIMING_
		call		COMTIMING_

		movf		ASCI2,w
		movwf		COMSEND
		call		COMSEND_

		call		COMTIMING_
		call		COMTIMING_
		call		COMTIMING_

		movf		ASCI1,w
		movwf		COMSEND
		call		COMSEND_

		call		COMTIMING_
		call		COMTIMING_
		call		COMTIMING_

		movf		ASCI0,w
		movwf		COMSEND
		call		COMSEND_

		call		COMTIMING_
		call		COMTIMING_
		call		COMTIMING_

		movlw		h'1B'           ;escape sequenz vt100
		movwf		COMSEND
		call		COMSEND_

		call		COMTIMING_
		call		COMTIMING_
		call		COMTIMING_

		movlw		h'5B'
		movwf		COMSEND
		call		COMSEND_

		call		COMTIMING_
		call		COMTIMING_
		call		COMTIMING_

		movlw		h'34'           ;'4' digits
		movwf		COMSEND
		call		COMSEND_

		call		COMTIMING_
		call		COMTIMING_
		call		COMTIMING_

		movlw		h'44'           ;'D' back
		movwf		COMSEND
		call		COMSEND_
 
                return


;------------------------------------------------------------------------------
;Cat:         SW Func / class com / COMINIT_
;------------------------------------------------------------------------------
;Level:       1
;------------------------------------------------------------------------------
;Task:        init com pin
;------------------------------------------------------------------------------
;Functions:     COMINIT_
;Based on:      no
;------------------------------------------------------------------------------
;Vars:          no
;Internal:      no
;------------------------------------------------------------------------------
;Bits:          no
;Internal:      no
;------------------------------------------------------------------------------
;Defines:       COMTX
;Internal:      no
;---------------|---------------|---------------|------------------------------
COMINIT_
                bsf             COMTX
                return

;------------------------------------------------------------------------------
;Cat:         SW Func / class com / COMSEND_
;------------------------------------------------------------------------------
;Level:       2
;------------------------------------------------------------------------------
;Task:        send one byte (emulate com port TX)
;------------------------------------------------------------------------------
;Functions:     COMSEND_
;Based on:      COMTIMING_
;------------------------------------------------------------------------------
;Vars:          COMSEND         ;byte to send
;Internal:    
;------------------------------------------------------------------------------
;Bits:          no              
;Internal:      no
;------------------------------------------------------------------------------
;Pindefines:    COMTX           ;tx pin 
;Internal:      no
;---------------|---------------|---------------|------------------------------
COMSEND_	
		bcf		COMTX           ;start

		call		COMTIMING_

		btfss		COMSEND,0
		goto		_S0L
		bsf		COMTX		;0
		goto		_S0H
_S0L		bcf		COMTX
_S0H
		call		COMTIMING_

		btfss		COMSEND,1
		goto		_S1L
		bsf		COMTX		;1
		goto		_S1H
_S1L		bcf		COMTX
_S1H

		call		COMTIMING_

		btfss		COMSEND,2
		goto		_S2L
		bsf		COMTX		;2
		goto		_S2H
_S2L		bcf		COMTX
_S2H
		call		COMTIMING_

		btfss		COMSEND,3
		goto		_S3L
		bsf		COMTX		;3
		goto		_S3H
_S3L		bcf		COMTX
_S3H

		call		COMTIMING_

		btfss		COMSEND,4
		goto		_S4L
		bsf		COMTX		;4
		goto		_S4H
_S4L		bcf		COMTX
_S4H		
		call		COMTIMING_


		btfss		COMSEND,5
		goto		_S5L
		bsf		COMTX		;5
		goto		_S5H
_S5L		bcf		COMTX
_S5H

		call		COMTIMING_

		btfss		COMSEND,6
		goto		_S6L
		bsf		COMTX		;6
		goto		_S6H
_S6L		bcf		COMTX
_S6H
		call		COMTIMING_


		btfss		COMSEND,7
		goto		_S7L
		bsf		COMTX		;7
		goto		_S7H
_S7L		bcf		COMTX
_S7H
		call		COMTIMING_

		bsf		COMTX		;stop

		call		COMTIMING_
		
		return
;------------------------------------------------------------------------------
;Cat:         SW Func / class com / COMTIMING_
;------------------------------------------------------------------------------
;Level:       1
;------------------------------------------------------------------------------
;Task:        emulate com timing @ 9600Baud (4Mhz)
;------------------------------------------------------------------------------
;Functions:     COMSEND_
;Based on:      no
;------------------------------------------------------------------------------
;Vars:          no
;Internal:      COMCNT
;------------------------------------------------------------------------------
;Bits:          no
;Internal:      no
;------------------------------------------------------------------------------
;Defines:       COMSTIME_D
;Internal:      no
;---------------|---------------|---------------|------------------------------
COMTIMING_
		movlw		COMSTIME_D
		movwf		COMCNT
_COMSWT1	decfsz		COMCNT,f
		goto		_COMSWT1
		return


;******************************************************************************
;*************************************************************** CLASS xxx



;------------------------------------------------------------------------------
;Cat:         SW Func / class xx / func xxx
;------------------------------------------------------------------------------
;Level:       x
;------------------------------------------------------------------------------
;Task:        
;------------------------------------------------------------------------------
;Functions:
;  USE:
;    Public:      no
;                 
;                 
;                 
;    Private:     no
;  BASE:
;    Public:      no
;                 
;                 
;                 
;                 
;                 
;    Private:     no
;------------------------------------------------------------------------------
;Vars:            no
;Internal:        no    
;------------------------------------------------------------------------------
;Bits:
;  USE:
;    Public:      no                
;    Private:     no           
;  BASE:          
;    Public:      no
;    Private:     no
;                 no
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------


;*********************************************************** END CLASS xxx
;******************************************************************************



;*************************************************************** HARDWARE LAYER
;******************************************************************************
;------------------------------------------------------------------------------
;Cat:         SW Func / class xxx /
;------------------------------------------------------------------------------
;Level:       x
;------------------------------------------------------------------------------
;Task:        
;------------------------------------------------------------------------------
;Functions:   
;Based on:    
;------------------------------------------------------------------------------
;Vars:        
;Internal:    
;------------------------------------------------------------------------------
;Bits:        
;Internal:    
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------
HWFUNC_
;                goto            BHWFUNC_


;------------------------------------------------------------------------------
;Cat:         SW Func
;------------------------------------------------------------------------------
;Level:       2
;------------------------------------------------------------------------------
;Task:        translate data from ascii to 7segm & sort by seg connection
;------------------------------------------------------------------------------
;Functions:   _7SEG                             ;array of 7seg data
;------------------------------------------------------------------------------
;Vars:        VAL                               ;input   ascii
;             DEC7S                             ;output  7seg
;------------------------------------------------------------------------------
;Bits:        no
;------------------------------------------------------------------------------
;---------------|---------------|---------------|------------------------------
   
DECODE_ 
                ;addlw           h'30'           ;sel ascii
		
                ;addlw           h'80'           ;+if called with ascii >7F hex
                ;btfss           STATUS,0
                ;addlw           h'80'           ;-if called with ascii >7F hex
                movwf		VAL
                call            _7SEG            ;get 7 seg value 
                movwf           VAL

                clrf            DEC7S

                bcf             DEC7S,4
                btfsc           VAL,1
                bsf             DEC7S,4

                bcf             DEC7S,6
                btfsc           VAL,2
                bsf             DEC7S,6

                bcf             DEC7S,7
                btfsc           VAL,3
                bsf             DEC7S,7

                bcf             DEC7S,1
                btfsc           VAL,4
                bsf             DEC7S,1

                bcf             DEC7S,0
                btfsc           VAL,5
                bsf             DEC7S,0

                bcf             DEC7S,2
                btfsc           VAL,6
                bsf             DEC7S,2

                bcf             DEC7S,3
                btfsc           VAL,7
                bsf             DEC7S,3
       
                movf            DEC7S,w   
       
                return



;******************************************************************************
;*********************************************************** END HARDWARE LAYER

;******************************************************************************
;************************************************************* END OF FUNCTIONS

;************************************************************ INTERRUPT HANDLER
;******************************************************************************

;********************************************************** INTERRUPT FUNCTIONS
;******************************************************************************


_ISR								;
                PUSH_


		bcf		INTCON,2			;resets interrupt bit

		decfsz		ISR_TMR,f			;prescaler (/256)
		goto		_ISRNOSEC

		bsf		F_CLKTRIG       		;bit ist zwischenhndler zwischen interrupt und main
                

_ISRNOSEC	

                movf            ISR_TMR,w
                
                btfsc           STATUS,2
                bsf             F_DISPTRIG

                addlw           h'80'

                btfsc           STATUS,2
                bsf             F_DISPTRIG


                POP_

		retfie

;******************************************************************************
;*************************************************** END OF INTERRUPT FUNCTIONS

;***************************************************************** Look Up ****
;******************************************************************************


;********************************Mux Bright************************************

		org		0x038F		
_BRI
		movlw		h'03'
		movwf		PCLATH
		movf		VAL,w
		addwf		PCL,f

		retlw		h'00'		
		retlw		h'01'
		retlw		h'11'
		retlw		h'92'
		retlw		h'aa'
		retlw		h'da'
		retlw		h'ee'
		retlw		h'ff'


;********************************7seg decoder**********************************
;
;       Bit0 db/nc
;       Bit1 c
;       Bit2 d
;       Bit3 e
;       
;       Bit4 g
;       Bit5 f
;       Bit6 a
;       Bit7 b
;       
;
;******************************************************************************

		org		0x039B		
_7SEG
		movlw		h'03'
		movwf		PCLATH
		movf		VAL,w
                addlw           h'E0'
		addwf		PCL,f

						;asci 20

		retlw		h'00'		;Leerzeichen		
		retlw		h'83'		;!
		retlw		h'A0'		;"
		retlw		h'54'		;#
		retlw		h'72'		;$
		retlw		h'66'		;%
		retlw		h'FF'		;&
		retlw		h'80'		;'
		retlw		h'6C'		;( = [
		retlw		h'C6'		;) = ]
		retlw		h'BA'		;* = H
		retlw		h'92'		;+ = {
		retlw		h'02'		;,
		retlw		h'10'		;-
		retlw		h'01'		;.
		retlw		h'98'		;/ 

						;asci 30

		retlw		h'EE'		;0		
		retlw		h'82'		;1
		retlw		h'DC'		;2
		retlw		h'D6'		;3
		retlw		h'B2'		;4
		retlw		h'76'		;5
		retlw		h'7E'		;6
		retlw		h'C2'		;7
		retlw		h'FE'		;8
		retlw		h'F6'		;9
		retlw		h'50'		;:
		retlw		h'51'		;;
		retlw		h'1C'		;<
		retlw		h'50'		;==:
		retlw		h'16'		;>
		retlw		h'D9'		;?

						;asci 40

		retlw		h'00'		;@
		retlw		h'FA'		;A
		retlw		h'FE'		;B=8
		retlw		h'6C'		;C
		retlw		h'C6'		;D
		retlw		h'7C'		;E
		retlw		h'78'		;F
		retlw		h'6E'		;G
		retlw		h'BA'		;H
		retlw		h'28'		;I
		retlw		h'8E'		;J
		retlw		h'BB'		;K
		retlw		h'2C'		;L
		retlw		h'EB'		;M
		retlw		h'EA'		;N
		retlw		h'EE'		;O=0
	
						;asci 50

		retlw		h'F8'		;P
		retlw		h'EF'		;Q
		retlw		h'F9'		;R
		retlw		h'76'		;S=5
		retlw		h'68'		;T
		retlw		h'AE'		;U
		retlw		h'A8'		;V
		retlw		h'AA'		;W
		retlw		h'32'		;X =\
		retlw		h'B6'		;Y
		retlw		h'CC'		;Z
		retlw		h'6C'		;[
		retlw		h'F4'		;ungleich
		retlw		h'C6'		;]
		retlw		h'E0'		;^
		retlw		h'04'		;_

						;asci 60

		retlw		h'C0'		;`		
		retlw		h'1F'		;a
		retlw		h'3E'		;b
		retlw		h'1C'		;c=<
		retlw		h'9E'		;d
		retlw		h'FC'		;e
		retlw		h'E8'		;f
		retlw		h'F6'		;g=9
		retlw		h'3A'		;h
		retlw		h'0C'		;i
		retlw		h'86'		;j
		retlw		h'3B'		;k
		retlw		h'28'		;l
		retlw		h'1B'		;m
		retlw		h'1A'		;n
		retlw		h'1E'		;o
	
						;asci 70

		retlw		h'9A'		;p		
		retlw		h'F2'		;q
		retlw		h'18'		;r
		retlw		h'36'		;s
		retlw		h'3C'		;t
		retlw		h'0E'		;u
		retlw		h'0A'		;v
		retlw		h'0F'		;w
		retlw		h'22'		;x
		retlw		h'B2'		;y=4
		retlw		h'14'		;z
		retlw		h'38'		;{=+
		retlw		h'82'		;|=1
		retlw		h'92'		;}
		retlw		h'E0'		;->
		retlw		h'70'		;<-


;********************************************************** end of 7 seg decode
;******************************************************************************

END

